#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------------------------------------------------
# Author: Lalith Kumar Shiyam Sundar
#         Sebastian Gutschmayer
# Institution: Medical University of Vienna
# Research Group: Quantitative Imaging and Medical Physics (QIMP) Team
# Date: 09.02.2023
# Version: 2.0.0
#
# Description:
# This module handles image conversion for the moosez.
#
# Usage:
# The functions in this module can be imported and used in other modules within the moosez to perform image conversion.
#
# ----------------------------------------------------------------------------------------------------------------------

import logging
import re
import subprocess
import os
from halo import Halo
import SimpleITK
import pydicom
from moosez import constants
from multiprocessing import Pool
from typing import List
from tqdm import tqdm
from rich.progress import Progress



def read_dicom_folder(folder_path: str) -> SimpleITK.Image:
    """
    Reads a folder of DICOM files and returns the image

    :param folder_path: str, Directory to get DICOM files from
    """
    reader = SimpleITK.ImageSeriesReader()
    dicom_names = reader.GetGDCMSeriesFileNames(folder_path)
    reader.SetFileNames(dicom_names)

    dicom_image = reader.Execute()
    return dicom_image


def non_nifti_to_nifti(input_path: str, output_directory: str = None) -> None:
    """
    Converts any image format known to ITK to NIFTI

    :param input_path: str, Directory OR filename to convert to nii.gz
    :param output_directory: str, optional output directory to write the image to.
    """
    subject_name = os.path.basename(os.path.dirname(input_path))
    output_image_basename = "output"
    output_image = None  # initialize output_image
    if os.path.isdir(input_path):
        image_probe = os.listdir(input_path)[0]
        modality_tag = pydicom.read_file(os.path.join(input_path, image_probe)).Modality
        if image_probe.endswith(('IMA', 'dcm')):
            output_image = read_dicom_folder(input_path)
            if modality_tag == 'PT':
                output_image_basename = f"{constants.TRACER_FDG}_PET_{subject_name}.nii"
            elif modality_tag == 'CT':
                output_image_basename = f"{modality_tag}_{subject_name}.nii"
    elif os.path.isfile(input_path):
        if input_path.endswith('.nii.gz') or input_path.endswith('.nii'):
            return
        output_image = SimpleITK.ReadImage(input_path)
        output_image_basename = f"{os.path.splitext(os.path.basename(input_path))[0]}.nii"
    else:
        return

    if output_directory is None:
        output_directory = os.path.dirname(input_path)
    output_image_path = os.path.join(output_directory, output_image_basename)
    SimpleITK.WriteImage(output_image, output_image_path)


def standardize_to_nifti(parent_dir: str):
    """
    Converts all images in a parent directory to NIFTI
    """
    # go through the subdirectories
    subjects = os.listdir(parent_dir)
    # get only the directories
    subjects = [subject for subject in subjects if os.path.isdir(os.path.join(parent_dir, subject))]

    with Progress() as progress:
        task = progress.add_task("[cyan] Processing subjects...", total=len(subjects))
        for subject in subjects:
            subject_path = os.path.join(parent_dir, subject)
            if os.path.isdir(subject_path):
                images = os.listdir(subject_path)
                for image in images:
                    if os.path.isdir(os.path.join(subject_path, image)):
                        image_path = os.path.join(subject_path, image)
                        non_nifti_to_nifti(image_path)
                    elif os.path.isfile(os.path.join(subject_path, image)):
                        image_path = os.path.join(subject_path, image)
                        non_nifti_to_nifti(image_path)
            else:
                continue
            progress.update(task, advance=1, description=f"[cyan] Processing {subject}...")





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------------------------------------------------
# Author: Lalith Kumar Shiyam Sundar
#         Sebastian Gutschmayer
# Institution: Medical University of Vienna
# Research Group: Quantitative Imaging and Medical Physics (QIMP) Team
# Date: 05.06.2023
# Version: 2.0.0
#
# Description:
# This module handles image processing for the moosez.
#
# Usage:
# The functions in this module can be imported and used in other modules within the moosez for image processing.
#
# ----------------------------------------------------------------------------------------------------------------------

import numpy as np
import nibabel
import SimpleITK


def resample(input_image_path: str, output_image_path: str, interpolation: str, desired_spacing: list) -> str:
    """
    Resamples an image to a new spacing.
    :param input_image_path: Path to the input image.
    :param output_image_path: Path to the output image.
    :param interpolation: The interpolation method to use.
    :param desired_spacing: The new spacing to use.
    :return: The path to the output image.
    """

    # Set interpolator for SimpleITK

    if interpolation == 'nearest':
        interpolation_method = SimpleITK.sitkNearestNeighbor
    elif interpolation == 'linear':
        interpolation_method = SimpleITK.sitkLinear
    else:
        raise ValueError('The interpolation method is not supported.')

    # Load the image and get necessary information
    input_image = nibabel.load(input_image_path)
    image_data = input_image.get_fdata()
    image_header = input_image.header
    image_affine = input_image.affine
    original_spacing = image_header.get_zooms()
    original_size = image_data.shape
    translation_vector = image_affine[:3, 3]
    rotation_matrix = image_affine[:3, :3]

    # Convert to SimpleITK image format
    image_data_swapped_axes = image_data.swapaxes(0, 2)
    sitk_input_image = SimpleITK.GetImageFromArray(image_data_swapped_axes)
    sitk_input_image.SetSpacing([spacing.item() for spacing in original_spacing])
    axis_flip_matrix = np.diag([-1, -1, 1])
    sitk_input_image.SetOrigin(np.dot(axis_flip_matrix, translation_vector))
    sitk_input_image.SetDirection((np.dot(axis_flip_matrix, rotation_matrix) / np.absolute(original_spacing)).ravel())

    # Resample the image to the desired spacing
    desired_spacing = np.array(desired_spacing)
    new_size = [round(original_size[i] * (original_spacing[i] / desired_spacing[i])) for i in range(len(original_size))]
    resampled_sitk_image = SimpleITK.Resample(sitk_input_image, new_size, SimpleITK.Transform(), interpolation_method,
                                              sitk_input_image.GetOrigin(), desired_spacing.tolist(),
                                              sitk_input_image.GetDirection(), 0.0, sitk_input_image.GetPixelIDValue())

    # Save the resampled image to disk
    SimpleITK.WriteImage(resampled_sitk_image, output_image_path)

    return output_image_path


def get_spacing(image_path: str) -> list:
    """
    Returns the spacing of an image.
    :param image_path: Path to the image.
    :return: The spacing of the image.
    """
    image = nibabel.load(image_path)
    image_header = image.header
    spacing = image_header.get_zooms()
    return [spacing.item() for spacing in spacing]



from setuptools import setup, find_packages


setup(
    name='moosez',
    version='2.0.10',
    author='Lalith Kumar Shiyam Sundar',
    author_email='Lalith.shiyamsundar@meduniwien.ac.at',
    description='An AI-inference engine for 3D clinical and preclinical whole-body segmentation tasks',
    python_requires='>=3.9',
    long_description='mooseZ is an AI-inference engine based on nnUNet, designed for 3D clinical and preclinical'
                     ' whole-body segmentation tasks. It serves models tailored towards different modalities such'
                     ' as PET, CT, and MR. mooseZ provides fast and accurate segmentation results, making it a '
                     'reliable tool for medical imaging applications.',
    url='https://github.com/QIMP-Team/mooseZ',
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
    ],

    keywords='moosez model-zoo nnUNet medical-imaging tumor-segmentation organ-segmentation bone-segmentation'
             ' lung-segmentation muscle-segmentation fat-segmentation vessel-segmentation'
             ' vertebral-segmentation rib-segmentation'
             ' preclinical-segmentation clinical-segmentation',
    packages=find_packages(),
    install_requires=[
        'nnunetv2',
        'nibabel~=3.2.2',
        'halo~=0.0.31',
        'pandas~=1.4.1',
        'SimpleITK~=2.2.1',
        'pydicom~=2.2.2',
        'argparse~=1.4.0',
        'imageio~=2.16.1',
        'numpy~=1.22.3',
        'mpire~=2.3.3',
        'openpyxl~=3.0.9',
        'matplotlib',
        'pyfiglet~=0.8.post1',
        'natsort~=8.1.0',
        'pillow>=9.2.0',
        'colorama~=0.4.6',
        'rich'
    ],
    entry_points={
        'console_scripts': [
            'moosez=moosez.moosez:main',
        ],
    },
)
